@import url('https://fonts.googleapis.com/css?family=Exo+2:300,300i,500,500i');

:root {
  --hover: rgba(200,200,200,1);
  --hover-controls: rgba(255,255,255,0.1);
  --green-color: #3b7538;
  --orange-color: #f26522;
  --blue-color: #0054a6;
}

body, html {
  height: 100%;
}

#app {
  font-family: 'Exo 2', sans-serif;
  font-weight: 300;
  font-size: 16px;
  margin: 0;
  padding-bottom: 10px;
  background-image: url(../imgs/arenas/DFHStadium.jpg);
  background-repeat: no-repeat;
  background-position: center;
  background-size: cover;
  transition: background-image 0.5s ease-in-out;
  min-height: 100%;
}

.rlbot-main-config .card-body, .settings-card .card-body {
  padding: 5px;
}

.navbar {
  background-color: rgb(17, 18, 19);
}

.control-bar {
  background-color: var(--green-color);
  height: 2.5rem;
  filter: drop-shadow(0 0 0.5rem rgba(0,0,0,1.0));
  margin-bottom: 30px;
}

.logo {
  width: 3rem;
  margin-top: -30px;
  margin-bottom: -25px;
  filter: drop-shadow(0 0 0.5rem rgba(0,0,0,0.6));
}

.rlbot-brand {
  color: white;
  font-size: 1.5rem;
  font-weight: bold;
}

.rlbot-card-header {
  font-size: 1.3rem;
  margin-right: 20px;
  font-weight: bold;
}

.team-entries {
  min-height: 160px;
  padding: 5px;
  margin-bottom: 20px;
}

.team-card {
  position: relative;
}

.team-label {
  position: absolute;
  bottom: 3px;
  color: white;
  opacity: 0.5;
}

.draggable {
  cursor: move;
}

.bot-card {
  margin: 2px;
  box-shadow: 1px 1px 7px #0000002e;
}

.bot-card .card-body {
  padding: 2px 5px;
  display: flex;
  width: 100%;
  align-items: center;
}

.unique-bot-identifier {
  color: #868686;
}

.script-card {
  background: linear-gradient(90deg, rgb(255, 255, 255) 0%, rgba(255, 255, 255, 0.34) 50%)
}

.center-flex {
  display: flex;
  align-items: center;
}

.bot-name {
  flex-grow: 1;
}

.bot-name:last-child {
  padding-right: 10px;
}

#app .secret-button {
  background: none;
  color: inherit;
  border: none;
  padding: 0;
  font: inherit;
  cursor: inherit;
  outline: none;
}

.secret-button:focus {
  background-color: rgba(0, 0, 0, 0.07);
}

#teamSwitcher .md-switch-thumb {
  background-color: var(--blue-color);
}

#teamSwitcher .md-switch-container {
  background-color: rgba(0, 84, 166, 0.5);
}

#teamSwitcher .orange .md-switch-thumb {
  background-color: var(--orange-color);
}

#teamSwitcher .orange .md-switch-container {
  background-color: rgba(242, 101, 34, 0.5);
}

#teamSwitcher .md-switch-container:focus-within {
  background-color: var(--hover)
}

.card.bot-pool, .card.team-card, .card.settings-card {
  margin: 10px;
  color: #333333;
}

#app .card .md-card-header {
  padding: 5px;
}

#app .bot-pool, #app .settings-card {
  background-color: #eee;
  padding: 2px 5px;
}

.bot-pool .bot-card {
  display: inline-flex;
}

.bot-card img {
  height: 1.5rem;
  margin: 3px;
}

.script-card img {
  margin: 0 3px 0 0;
}

.filtered {
  opacity: 0.2;
}

.bot-card img.darkened {
  filter: brightness(50%);
}

.org .bot-card img.darkened {
  filter: brightness(0%) invert(50%) sepia(25%) saturate(7482%) hue-rotate(350deg) brightness(102%) contrast(90%);
}

.blu .bot-card img.darkened {
  filter: brightness(0%) invert(17%) sepia(78%) saturate(3372%) hue-rotate(198deg) brightness(96%) contrast(101%);
}

.org .bot-card {
  color: var(--orange-color);
}

.blu .bot-card {
  color: var(--blue-color);
}

#app .org {
  color: var(--orange-color);
  background-color: var(--orange-color);
}

#app .blu {
  color: var(--blue-color);
  background-color: var(--blue-color);
}

.md-menu-content {
  z-index: 10;
}

.bot-card .bot-hover-reveal {
  filter: opacity(20%);
}

.bot-card:hover .bot-hover-reveal {
  filter: opacity(100%);
}

.bot-info-key {
  font-weight: bold;
}

#app .warning-icon, #app .warning-icon .md-icon {
  color: #f5b700;
}

button.icon-button {
  border: none;
  margin-left: 6px;
}

#rlbotgui .bot-pool-adder {
  vertical-align: top;
  margin-left: 20px;
  margin-top: 5px;
}

.bot-file-path {
  color: #939393;
  font-weight: normal;
}

.bot-logo {
  float:right;
  margin-left: 10px;
  margin-top: 15px;
  margin-bottom: 10px;
  max-width: 250px;
}

.md-dialog{
  width:80%;
}

.arena-background {
  background: url('/imgs/arena_diagram.png');
  background-size: contain;
  background-repeat: no-repeat;
  margin: 10px;
}

.btn.btn-secondary {
  background-color: white;
  color: black;
  border-color: #aeaeae;
  box-shadow: 0 2px 0 #989898cf;
}

.btn.btn-secondary:hover {
  color:black;
}

.btn.btn-primary {
  box-shadow: 0 2px 0 rgba(0, 3, 49, 0.8);
}

.btn.btn-secondary:hover {
  background-color: #d9d9d9;
}

.btn.btn-success {
  box-shadow: 0 3px 0 green;
}

.navbar-nav {
  align-items: center;
}

.form-group-inline .col {
  display: flex;
}

.form-group-inline .form-control {
  display: inline-block;
  width: initial;
}

.platform-icon {
  height: 1em;
}

.btn-dark .platform-icon {
  filter: invert()
}

.highlighted-mutator-field {
  font-weight: bold;
  color: rgb(27, 27, 27);
}